import { Injectable, Inject, PLATFORM_ID, ɵɵdefineInjectable, ɵɵinject, Component, ViewEncapsulation, Input, NgModule } from '@angular/core';
import { isPlatformBrowser, CommonModule } from '@angular/common';
import { Subject, of, timer, combineLatest } from 'rxjs';
import { switchMap, startWith, shareReplay, map, take, tap } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * Generated from: loading-bar.state.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function ILoadingBarState() { }
if (false) {
    /** @type {?} */
    ILoadingBarState.prototype.action;
    /** @type {?} */
    ILoadingBarState.prototype.value;
    /** @type {?} */
    ILoadingBarState.prototype.initialValue;
}
class LoadingBarState {
    constructor() {
        this.state = {
            action: null,
            value: 0,
            initialValue: 0,
        };
        this.requests = null;
        this.disabled = false;
        this.stream$ = new Subject();
        this.value$ = this.stream$.asObservable().pipe(switchMap((/**
         * @param {?} s
         * @return {?}
         */
        (s) => this.timer$(s))), startWith({ action: null, value: 0 }), shareReplay(), map((/**
         * @param {?} s
         * @return {?}
         */
        s => s.value)));
        this.timer$ = (/**
         * @param {?} s
         * @return {?}
         */
        (s) => {
            /** @type {?} */
            let state$ = of(s);
            switch (s.action) {
                case 'start':
                case 'increment':
                case 'set': {
                    if (this.requests > 0) {
                        state$ = timer(0, 250).pipe(map((/**
                         * @param {?} t
                         * @return {?}
                         */
                        t => t === 0
                            ? Object.assign({}, s, { value: this.state.value || s.initialValue }) : Object.assign({}, s, { value: this._increment() }))));
                    }
                    break;
                }
                case 'complete':
                case 'stop': {
                    // Attempt to aggregate any start/complete calls within 500ms:
                    state$ = s.value === 0 ? of(Object.assign({}, s)) : timer(0, 500).pipe(take(2), map((/**
                     * @param {?} t
                     * @return {?}
                     */
                    t => ({ value: t === 0 ? 100 : 0 }))));
                    break;
                }
            }
            return state$.pipe(map((/**
             * @param {?} next
             * @return {?}
             */
            next => (/** @type {?} */ ((Object.assign({}, next, { action: 'set' })))))), tap((/**
             * @param {?} next
             * @return {?}
             */
            (next) => this.next(next, false))));
        });
    }
    /**
     * @param {?=} initialValue
     * @return {?}
     */
    start(initialValue = 2) {
        if (this.disabled) {
            return;
        }
        this.next({ action: 'start', initialValue });
    }
    /**
     * @return {?}
     */
    stop() {
        this.next({ action: 'stop' });
    }
    /**
     * @return {?}
     */
    complete() {
        this.next({ action: 'complete' });
    }
    /**
     * @return {?}
     */
    disable() {
        this.disabled = true;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set(value) {
        this.next({ action: 'set', value });
    }
    /**
     * @param {?=} value
     * @return {?}
     */
    increment(value = 0) {
        this.next({ action: 'increment', value });
    }
    /**
     * @private
     * @param {?} state
     * @param {?=} emitEvent
     * @return {?}
     */
    next(state, emitEvent = true) {
        switch (state.action) {
            case 'start':
                this.requests = (this.requests || 0) + 1;
                break;
            case 'complete':
                this.requests = (this.requests || 1) - 1;
                if (this.requests > 0) {
                    return;
                }
                break;
            case 'stop':
                this.requests = 0;
                break;
            default:
                if (state.action === 'increment') {
                    state.value = this._increment(state.value);
                }
                break;
        }
        /** @type {?} */
        const newState = Object.assign({}, this.state, { action: null }, state);
        this.state = newState;
        if (emitEvent) {
            this.stream$.next(this.state);
        }
    }
    /**
     * @private
     * @param {?=} rnd
     * @return {?}
     */
    _increment(rnd = 0) {
        /** @type {?} */
        const stat = this.state.value;
        if (stat >= 99) {
            rnd = 0;
        }
        if (rnd === 0) {
            if (stat >= 0 && stat < 25) {
                // Start out between 3 - 6% increments
                rnd = (Math.random() * (5 - 3 + 1) + 3);
            }
            else if (stat >= 25 && stat < 65) {
                // increment between 0 - 3%
                rnd = (Math.random() * 3);
            }
            else if (stat >= 65 && stat < 90) {
                // increment between 0 - 2%
                rnd = (Math.random() * 2);
            }
            else if (stat >= 90 && stat < 99) {
                // finally, increment it .5 %
                rnd = 0.5;
            }
            else {
                // after 99%, don't increment:
                rnd = 0;
            }
        }
        return rnd + stat;
    }
}
if (false) {
    /**
     * @type {?}
     * @private
     */
    LoadingBarState.prototype.state;
    /**
     * @type {?}
     * @private
     */
    LoadingBarState.prototype.requests;
    /**
     * @type {?}
     * @private
     */
    LoadingBarState.prototype.disabled;
    /**
     * @type {?}
     * @private
     */
    LoadingBarState.prototype.stream$;
    /** @type {?} */
    LoadingBarState.prototype.value$;
    /**
     * @type {?}
     * @private
     */
    LoadingBarState.prototype.timer$;
}

/**
 * @fileoverview added by tsickle
 * Generated from: loading-bar.service.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class LoadingBarService {
    /**
     * @param {?} platformId
     */
    constructor(platformId) {
        this.platformId = platformId;
        this.refs = {};
        this.streams$ = (new Subject());
        this.value$ = this.streams$.pipe(switchMap((/**
         * @param {?} s
         * @return {?}
         */
        s => combineLatest(...s))), map((/**
         * @param {?} v
         * @return {?}
         */
        v => Math.max(...v))));
    }
    /**
     * @deprecated use `value$` instead.
     * @return {?}
     */
    get progress$() { return this.value$; }
    /**
     * @deprecated use `useRef` instead.
     * @param {?=} initialValue
     * @return {?}
     */
    start(initialValue = 2) { this.useRef().start(initialValue); }
    /**
     * @deprecated use `useRef` instead.
     * @param {?} value
     * @return {?}
     */
    set(value) { this.useRef().set(value); }
    /**
     * @deprecated use `useRef` instead.
     * @param {?=} value
     * @return {?}
     */
    increment(value) { this.useRef().increment(value); }
    /**
     * @deprecated use `useRef` instead.
     * @return {?}
     */
    complete() { this.useRef().complete(); }
    /**
     * @deprecated use `useRef` instead.
     * @return {?}
     */
    stop() { this.useRef().stop(); }
    /**
     * @param {?=} id
     * @return {?}
     */
    useRef(id = 'default') {
        if (!this.refs[id]) {
            this.refs[id] = new LoadingBarState();
            this.streams$.next(Object.keys(this.refs)
                .map((/**
             * @param {?} s
             * @return {?}
             */
            s => this.refs[s].value$)));
            if (!isPlatformBrowser(this.platformId)) {
                this.refs[id].disable();
            }
        }
        return this.refs[id];
    }
}
LoadingBarService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */
LoadingBarService.ctorParameters = () => [
    { type: Object, decorators: [{ type: Inject, args: [PLATFORM_ID,] }] }
];
/** @nocollapse */ LoadingBarService.ngInjectableDef = ɵɵdefineInjectable({ factory: function LoadingBarService_Factory() { return new LoadingBarService(ɵɵinject(PLATFORM_ID)); }, token: LoadingBarService, providedIn: "root" });
if (false) {
    /**
     * @type {?}
     * @private
     */
    LoadingBarService.prototype.refs;
    /**
     * @type {?}
     * @private
     */
    LoadingBarService.prototype.streams$;
    /** @type {?} */
    LoadingBarService.prototype.value$;
    /**
     * @type {?}
     * @private
     */
    LoadingBarService.prototype.platformId;
}

/**
 * @fileoverview added by tsickle
 * Generated from: loading-bar.component.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class LoadingBarComponent {
    /**
     * @param {?} loader
     */
    constructor(loader) {
        this.loader = loader;
        this.includeSpinner = true;
        this.includeBar = true;
        this.fixed = true;
        this.value$ = this.loader.progress$;
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.ref) {
            this.value$ = this.ref
                ? this.loader.useRef(this.ref).value$
                : this.loader.value$;
        }
    }
}
LoadingBarComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngx-loading-bar',
                template: `
    <ng-container *ngIf="(value != null ? value : value$|async) as progress">
      <div id="loading-bar-spinner" *ngIf="includeSpinner" [style.color]="color">
        <div [style.width]="diameter" [style.height]="diameter" class="spinner-icon"></div>
      </div>
      <div id="loading-bar" *ngIf="includeBar" [style.color]="color">
        <div class="bar" [style.background]="color" [style.height]="height" [style.width]="progress + '%'">
          <div class="peg" [style.height]="height"></div>
        </div>
      </div>
    </ng-container>
  `,
                preserveWhitespaces: false,
                encapsulation: ViewEncapsulation.Emulated,
                host: {
                    '[class.loading-bar-fixed]': 'fixed',
                },
                styles: [":host{position:relative;display:block}:host.loading-bar-fixed>div .bar{position:fixed}:host.loading-bar-fixed>div#loading-bar-spinner{position:fixed;top:10px;left:10px}[dir=rtl] :host.loading-bar-fixed>div#loading-bar-spinner{right:10px;left:unset}:host.loading-bar-fixed>div .peg{display:block}:host>div{pointer-events:none;-webkit-transition:350ms linear;transition:350ms linear;color:#29d}:host>div .bar{-webkit-transition:width 350ms;transition:width 350ms;background:#29d;position:absolute;z-index:10002;top:0;left:0;width:100%;height:2px;border-bottom-right-radius:1px;border-top-right-radius:1px}[dir=rtl] :host>div .bar{right:0;left:unset}:host>div .peg{display:none;position:absolute;width:70px;right:0;top:0;height:2px;opacity:.45;box-shadow:1px 0 6px 1px;color:inherit;border-radius:100%}:host>div#loading-bar-spinner{display:block;position:absolute;z-index:10002;top:5px;left:0}:host>div#loading-bar-spinner .spinner-icon{width:14px;height:14px;border:2px solid transparent;border-top-color:inherit;border-left-color:inherit;border-radius:50%;-webkit-animation:.4s linear infinite loading-bar-spinner;animation:.4s linear infinite loading-bar-spinner}@-webkit-keyframes loading-bar-spinner{0%{-webkit-transform:rotate(0);transform:rotate(0)}100%{-webkit-transform:rotate(360deg);transform:rotate(360deg)}}@keyframes loading-bar-spinner{0%{-webkit-transform:rotate(0);transform:rotate(0)}100%{-webkit-transform:rotate(360deg);transform:rotate(360deg)}}"]
            }] }
];
/** @nocollapse */
LoadingBarComponent.ctorParameters = () => [
    { type: LoadingBarService }
];
LoadingBarComponent.propDecorators = {
    includeSpinner: [{ type: Input }],
    includeBar: [{ type: Input }],
    fixed: [{ type: Input }],
    color: [{ type: Input }],
    value: [{ type: Input }],
    ref: [{ type: Input }],
    height: [{ type: Input }],
    diameter: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    LoadingBarComponent.prototype.includeSpinner;
    /** @type {?} */
    LoadingBarComponent.prototype.includeBar;
    /** @type {?} */
    LoadingBarComponent.prototype.fixed;
    /** @type {?} */
    LoadingBarComponent.prototype.color;
    /** @type {?} */
    LoadingBarComponent.prototype.value;
    /** @type {?} */
    LoadingBarComponent.prototype.ref;
    /** @type {?} */
    LoadingBarComponent.prototype.height;
    /** @type {?} */
    LoadingBarComponent.prototype.diameter;
    /** @type {?} */
    LoadingBarComponent.prototype.value$;
    /** @type {?} */
    LoadingBarComponent.prototype.loader;
}

/**
 * @fileoverview added by tsickle
 * Generated from: core.module.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class LoadingBarModule {
}
LoadingBarModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                declarations: [LoadingBarComponent],
                exports: [LoadingBarComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * Generated from: public_api.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * Generated from: ngx-loading-bar-core.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { LoadingBarComponent, LoadingBarModule, LoadingBarService };
//# sourceMappingURL=ngx-loading-bar-core.js.map
